"""Magic function system for convenient interfacing with Gnuplot.

Warning: This module can ONLY be imported inside IPython or through an
embedded IPython instance.

This requires the Gnuplot.py module for interfacing python with gnuplot. It
can be downloaded from:

http://gnuplot-py.sourceforge.net/

This system builds the magic functions:

@gp -> pass one command to gnuplot and execute them or open a gnuplot shell
where each line of input is executed.

@gp_set_instance -> see below.

and global variables:

- g: pointing to a running gnuplot instance.
- Gnuplot: the Gnuplot module.

The docstrings below contain more details.

Please note that all commands are executed in the gnuplot namespace, so no
python variables exist there. Any data communication has to be done via files,
or using the methods of the gnuplot instance directly. For convenience, a copy
of the internal gnuplot instance called 'g' is created for you at startup. You
can call whichever methods you need on g (try typing g.<TAB> to see a list of
g's methods, or see the documentation for the Gnuplot.py module for details).

Configure this file's behavior by setting the global variable 'gnuplot_mouse'.

Inspired by a suggestion/request from Arnd Baecker."""

# If you have a mouse-enabled gnuplot, you can set gnuplot_mouse to 1,
# otherwise leave it at 0. Unfortunately there's no way to currently catch the
# resulting error message as a python exception if this doesn't work, so I
# can't leave it permanently on.

# Please note that this feature may cause some hiccups in communication with
# the gnuplot process (requiring sometimes a command to be issued twice). This
# is discussed in the gnuplot help and is an issue beyond my control. If it
# becomes a problem, resign yourself to not using the mouse features in
# gnuplot through IPython.

gnuplot_mouse = 0

# Create the Gnuplot instance we need
import Gnuplot

__IPYTHON__.gnuplot = Gnuplot.Gnuplot()
__IPYTHON__.gnuplot.shell_first_time = 1
g = __IPYTHON__.gnuplot  # alias for interactive convenience

# Force g as a global in IPython's namespace if we're being imported
if __name__ == 'GnuplotMagic':
    __IPYTHON__.user_ns['g'] = g
    __IPYTHON__.user_ns['Gnuplot'] = Gnuplot

if gnuplot_mouse:
    g('set mouse')
    print "*** g is a global alias to the internal Gnuplot instance (mouse enabled)."
else:
    print "*** g is a global alias to the internal Gnuplot instance."
print "*** Gnuplot instance access via @gp, or by calling methods of g directly."
print "*** Gnuplot module has been exported as a global."

# Define the magic functions for communicating with the above gnuplot instance.
def magic_gp(self,parameter_s=''):
    """Execute a gnuplot command or open a gnuplot shell.

    Usage (omit the @ if automagic is on). There are two ways to use it:

      1) @gp 'command' -> passes 'command' directly to the gnuplot instance.

      2) @gp -> will open up a prompt (gnuplot>) which takes input until '.'
      is entered (without quotes), or ^C or ^D are pressed. Each line of input
      is given to gnuplot as a command to be executed. If you need to type a
      multi-line command, use \\ at the end of each intermediate line.

      IPython traps the following forms of gnuplot's termination command:
      'quit' and 'exit', and simply returns to IPython without killing the
      gnuplot process. But if you type another form accepted by gnuplot,
      you'll terminate gnuplot (see below for how to restart it). So in order
      to simply return to IPython, use only one of the following exit options:
      '.', 'quit','exit', ^D (EOF) or ^C.

      Upon exiting of the gnuplot sub-shell, you return to your IPython
      session (the gnuplot sub-shell can be invoked as many times as needed).

    This system relies on the existence of a user-level variable called
    __gnuplot pointing to an active Gnuplot session. This variable is
    automatically created at startup, but if you overwrite it the system will
    stop functioning. You can create it again with a call like:

    In [1]: __gnuplot = Gnuplot.Gnuplot()    """

    from IPython.genutils import raw_input_ext

    if parameter_s.strip() == '':
        try:
            self.shell.gnuplot.has_run
        except AttributeError:
            print \
"""To exit gnuplot and return to IPython: '.', 'quit', 'exit', ^C or ^D (EOF).
Use \\ to break multi-line commands."""
            self.shell.gnuplot.has_run = 1
        try:
            cmd = raw_input_ext('gnuplot> ','more...> ')
            while cmd.strip() not in ['.','quit','exit']:
                self.shell.gnuplot(cmd)
                cmd = raw_input_ext('gnuplot> ','more...> ')
        except (EOFError,KeyboardInterrupt):
            pass
    else:
        self.shell.gnuplot(parameter_s)


def magic_gp_set_instance(self,parameter_s=''):
    """Set the global gnuplot instance accessed by the @gp magic function.

    @gp_set_instance name

    Call with the name of the new instance at the command line. If you want to
    set this instance in your own code (using an embedded IPython, for
    example), simply set the variable __IP.gnuplot to your own gnuplot
    instance object."""
    
    self.shell.gnuplot = eval(parameter_s)


# Add the new magic functions to the class dict
from IPython.iplib import InteractiveShell
InteractiveShell.magic_gp = magic_gp
InteractiveShell.magic_gp_set_instance = magic_gp_set_instance

# Keep global namespace clean
del magic_gp,magic_gp_set_instance,gnuplot_mouse

#********************** End of file <GnuplotMagic.py> *********************
